@@ -3,9 +3,9 @@
  *
  * Get the canonicalized absolute pathnames. The basis for SAKURA and TOMOYO.
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.6.7+   2009/04/10
+ * Version: 1.6.9+   2012/05/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -373,6 +373,56 @@ int ccs_realpath_both(const char *pathname, struct ccs_execve_entry *ee)
 	return ret ? -ENOMEM : 0;
 }
 
+/**
+ * ccs_encode: Encode binary string to ascii string.
+ *
+ * @str: String in binary format.
+ *
+ * Returns pointer to @str in ascii format on success, NULL otherwise.
+ *
+ * This function uses ccs_alloc(), so caller must ccs_free() if this function
+ * didn't return NULL.
+ */
+char *ccs_encode(const char *str)
+{
+	int len = 0;
+	const char *p = str;
+	char *cp;
+	char *cp0;
+	if (!p)
+		return NULL;
+	while (*p) {
+		const unsigned char c = *p++;
+		if (c == '\\')
+			len += 2;
+		else if (c > ' ' && c < 127)
+			len++;
+		else
+			len += 4;
+	}
+	len++;
+	cp = ccs_alloc(len, false);
+	if (!cp)
+		return NULL;
+	cp0 = cp;
+	p = str;
+	while (*p) {
+		const unsigned char c = *p++;
+		if (c == '\\') {
+			*cp++ = '\\';
+			*cp++ = '\\';
+		} else if (c > ' ' && c < 127) {
+			*cp++ = c;
+		} else {
+			*cp++ = '\\';
+			*cp++ = (c >> 6) + '0';
+			*cp++ = ((c >> 3) & 7) + '0';
+			*cp++ = (c & 7) + '0';
+		}
+	}
+	return cp0;
+}
+
 /**
  * ccs_round_up - Round up an integer so that the returned pointers are appropriately aligned.
  *
@@ -644,7 +694,8 @@ static int ccs_round2(size_t size)
 /**
  * ccs_alloc - Allocate memory for temporary purpose.
  *
- * @size: Size in bytes.
+ * @size:        Size in bytes.
+ * @check_quota: True to check memory quota, false otherwise.
  *
  * Returns pointer to allocated memory on success, NULL otherwise.
  */
